// Bibliotheken für das Display
#include <LiquidCrystal_I2C.h>  // LiquidCrystal_I2C Bibliothek einbinden.
#include <Wire.h>               // Wire Bibliothek einbinden.

// Bibliotheken für die SD-Karte
#include <SD.h>
#include <SPI.h>

// Bibliothek für den CO2 Seensor
#include <MHZ19.h>
#include <SoftwareSerial.h>

// Pins vom CO2 Sensor
int MHZ_TX_PIN = 5;  // TX Pin von MH-Z19B
int MHZ_RX_PIN = 6;  // RX Pin von MH-Z19B

// Objekt für den Sensor
MHZ19 co2Sensor;
// Objekt für die Kommunikation mit dem Sensor.
// Verbine Arduino RX mit MH-Z19B TX und
// Arduino TX mit MH-Z19B RX.
SoftwareSerial co2Serial(MHZ_TX_PIN, MHZ_RX_PIN);

// Objekt für das LCD Display
LiquidCrystal_I2C lcd(0x27, 16, 2);

// Anfang des Dateinamens für die Dateien auf der SD-Karte
// Datei hat dann z.B. den Namen "CO2_0.CSV"
// Der Name darf insgesammt nicht mehr als 8 Zeichen beinhalten.
String logPrefix = "CO2_";
// Name der Datei zum abspeichern der Datei
String logFileName;

void setup() {
  // Starte serielle Schnittstelle zur Kommunikation mit dem Arduino
  Serial.begin(9600);

  // Starte Kommunikationsschnittstelle für den Co2 Sensor
  co2Serial.begin(9600);
  // Starte den Sensor mit der Kommunikationsschnittstelle
  co2Sensor.begin(co2Serial);
  // Schalte automatische Kalibration ab
  co2Sensor.autoCalibration(false);

  // Starte das Display
  lcd.init();
  // Hintergrundbeleuchtung fürs Display einschalten
  lcd.backlight();

  // Warte solange bis eine SD-Karte eingesteckt wurde
  while (!SD.begin(SS_PIN)) {
    Serial.println(F("Konnte keine Verbindung zur SD-Karte aufbauen."));
    Serial.println(F("Versuche es in 2 Sekunden nochmal"));
    delay(2000);
  }
  // Finde Dateinamen der frei ist.
  int number = 0;
  while (SD.exists(logPrefix + number + ".CSV")) {
    number++;
  }

  logFileName = logPrefix + number + ".CSV";  // Setze den Namen zusammen
  Serial.print(F("Setup erfolgreich. Speichere Daten in: "));
  Serial.println(logFileName);

  Serial.println(F("Warte 30 Sekunden damit der CO2 Sensor startet"));
  delay(30000);
}

void loop() {
  // Warte 10 Sekunden
  delay(10000);

  // Hole den CO2 Gehalt vom Sensor in ppm
  int co2Content = co2Sensor.getCO2();

  // Gebe den CO2 Gehalt auf der seriellen Schnittstelle aus
  Serial.println(co2Content);

  // Gebe den CO2 Gehalt auf dem Display aus
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(F("CO2 Gehalt ppm:"));
  lcd.setCursor(0, 1);
  lcd.print(co2Content);

  //Öeffne die Datei auf der SD-Karte
  File logFile = SD.open(logFileName, FILE_WRITE);
  // Speichere den CO2 Gehalt mit der vergangenden Zeit auf der SD-Karte
  logFile.print(millis());
  logFile.print(F(","));
  logFile.println(co2Content);

  // Schliesse die Datei auf der SD-Karte
  logFile.close();
}
